# coding: utf-8
from django.db import models
from objectpack.exceptions import ValidationError

from educommon.django.db.models import BaseModel
from educommon.m3.extensions.listeners.delete_check.mixins import (
    CascadeDeleteMixin
)
from educommon.utils.misc import cached_property

from .base import ModelDataSource
from .registries import registry
from .validators import validate_data_source_name


# -----------------------------------------------------------------------------
class ReportTemplate(CascadeDeleteMixin, BaseModel):

    u"""Модель "Шаблон отчета"."""

    USER_DEFINED = 0
    EXCEL_SIMPLE = 1
    EXCEL_MERGED = 2
    FORMAT_CHOICES = (
        (USER_DEFINED, u'Спрашивать перед сборкой'),
        (EXCEL_SIMPLE, u'Excel (без объединения ячеек)'),
        (EXCEL_MERGED, u'Excel (с объединением ячеек)'),
    )

    title = models.CharField(
        u'Наименование шаблона отчета',
        max_length=1000,
        unique=True,
    )
    data_source_name = models.CharField(
        u'Имя источника данных',
        max_length=500,
        validators=[validate_data_source_name],
    )
    format = models.PositiveSmallIntegerField(
        u'Формат сборки',
        choices=FORMAT_CHOICES,
        default=USER_DEFINED,
    )

    class Meta:
        verbose_name = u'Отчет'
        verbose_name_plural = u'Отчеты'

    def simple_clean(self, errors):
        # ---------------------------------------------------------------------
        # Проверка уникальности наименования шаблона отчета.
        if self.title:
            query = ReportTemplate.objects.filter(
                title__smart_iexact=self.title,
            )
            if self.pk:
                query = query.exclude(pk=self.pk)

            if query.exists():
                errors['title'].append(
                    u'Шаблон отчета с подобным именем уже существует.'
                )
        # ---------------------------------------------------------------------
        super(ReportTemplate, self).simple_clean(errors)

    @cached_property
    def data_source(self):
        if self.data_source_name not in registry:
            raise ValidationError(
                u'В шаблоне отчета указан несуществующий источник данных ({}).'
                .format(self.data_source_name)
            )

        return ModelDataSource(self.data_source_name)
# -----------------------------------------------------------------------------


class ReportColumn(BaseModel):

    u"""Модель "Столбец отчета"."""

    # Направления сортировки.
    ASCENDING = 1
    DESCENDING = 2
    SORT_CHOICES = (
        (ASCENDING, u'По возрастанию'),
        (DESCENDING, u'По убыванию'),
    )

    report_template = models.ForeignKey(
        ReportTemplate,
        verbose_name=u'Отчет',
        related_name='columns',
    )
    name = models.CharField(
        u'Имя колонки в источнике данных',
        max_length=300,
    )
    index = models.PositiveSmallIntegerField(
        u'Порядковый номер',
    )
    title = models.CharField(
        u'Отображаемое имя',
        max_length=300,
        null=True, blank=True,
    )

    cascade_delete_for = (report_template,)

    class Meta:
        verbose_name = u'Столбец отчета'
        verbose_name_plural = u'Столбцы отчетов'
        unique_together = (
            ('report_template', 'name'),
        )
        ordering = (
            'index',
        )
